@@ -172,7 +172,6 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 {
   gboolean exit_slideshow = FALSE;
   qiv_image *q = data;
-  Window xwindow;
   int move_step;
   const char *mess[2]={ jcmd, NULL};
 
@@ -359,7 +358,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
             qiv_load_image(q);
           }
-	  break;
+          break;
         case GDK_SCROLL_DOWN:
           if (fullscreen) {
             if (ev->key.state & GDK_SHIFT_MASK) {
@@ -377,7 +376,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
             qiv_load_image(q);
           }
-	  break;
+          break;
         case GDK_SCROLL_LEFT:
           if (fullscreen) {
             if (ev->key.state & GDK_SHIFT_MASK) {
@@ -395,7 +394,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
             qiv_load_image(q);
           }
-	  break;
+          break;
         case GDK_SCROLL_RIGHT:
           if (fullscreen) {
             if (ev->key.state & GDK_SHIFT_MASK) {
@@ -413,14 +412,14 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
             qiv_load_image(q);
           }
-	  break;
+          break;
         default:
-	  break;
+          break;
       }
     break;
 
     case GDK_KEY_PRESS:
-
+    {
       exit_slideshow = TRUE;    /* Abort slideshow on any key by default */
       qiv_cancel_cursor_timeout(q);
    #ifdef DEBUG
@@ -483,23 +482,23 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
         }
         /* else record keystroke if not null */
         else if(ev->key.string && *(ev->key.string) != '\0') {
-	  if (jumping) {
+          if (jumping) {
             /* leave jumping mode on invalid input and process
-	     * input key as if it entered in non jumping mode */
-	    if((jidx == 0 &&
-	        (*(ev->key.string) != 'f' &&
-		 *(ev->key.string) != 'F' &&
-		 *(ev->key.string) != 'b' &&
-		 *(ev->key.string) != 'B' &&
-		 *(ev->key.string) != 't' &&
-		 *(ev->key.string) != 'T')) ||
-	       (jidx > 0 &&
-		(*(ev->key.string) < '0' || *(ev->key.string) > '9'))) {
-	      jumping = 0;
-	      qiv_handle_event(ev, data);
-	      return;
-	    }
-	  }
+             * input key as if it entered in non jumping mode */
+            if((jidx == 0 &&
+                (*(ev->key.string) != 'f' &&
+                 *(ev->key.string) != 'F' &&
+                 *(ev->key.string) != 'b' &&
+                 *(ev->key.string) != 'B' &&
+                 *(ev->key.string) != 't' &&
+                 *(ev->key.string) != 'T')) ||
+               (jidx > 0 &&
+                (*(ev->key.string) < '0' || *(ev->key.string) > '9'))) {
+              jumping = 0;
+              qiv_handle_event(ev, data);
+              return;
+            }
+          }
           jcmd[jidx++]=*(ev->key.string);
           jcmd[jidx] = '\0';
           if (extcommand)
@@ -509,10 +508,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
       } else {
         guint keyval = ev->key.keyval;
         if(vikeys) {
-               if (keyval == 'h') keyval = GDK_Left;
-          else if (keyval == 'j') keyval = GDK_Down;
-          else if (keyval == 'k') keyval = GDK_Up;
-          else if (keyval == 'l') keyval = GDK_Right;
+               if (keyval == 'h') keyval = GDK_KEY_Left;
+          else if (keyval == 'j') keyval = GDK_KEY_Down;
+          else if (keyval == 'k') keyval = GDK_KEY_Up;
+          else if (keyval == 'l') keyval = GDK_KEY_Right;
           else
             switch (keyval) {
               case 'H':
@@ -625,9 +624,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
               first=1;
               qiv_load_image(q);
             }
-            xwindow = GDK_WINDOW_XWINDOW(q->win);
-            XIconifyWindow(GDK_DISPLAY(), xwindow, DefaultScreen(GDK_DISPLAY()));
-            break;
+            gdk_window_iconify(q->win);
 
             /* Statusbar on/off  */
 
@@ -1002,18 +999,26 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             /* Flip horizontal */
 
           case 'h':
-            imlib_image_flip_horizontal();
+          {
+            GdkPixbuf* pb_flipped = gdk_pixbuf_flip(q->pb, TRUE);
+            g_object_unref(q->pb);
+            q->pb = pb_flipped;
             snprintf(infotext, sizeof infotext, "(Flipped horizontally)");
             update_image(q, REDRAW);
             break;
+          }
 
             /* Flip vertical */
 
           case 'v':
-            imlib_image_flip_vertical();
+          {
+            GdkPixbuf* pb_flipped = gdk_pixbuf_flip(q->pb, FALSE);
+            g_object_unref(q->pb);
+            q->pb = pb_flipped;
             snprintf(infotext, sizeof infotext, "(Flipped vertically)");
             update_image(q, REDRAW);
             break;
+          }
 
             /* Watch File (on/off) */
 
@@ -1031,7 +1036,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             /* Rotate right */
 
           case 'k':
-            imlib_image_orientate(1);
+          {
+            GdkPixbuf* pb_rotate = gdk_pixbuf_rotate_simple(q->pb, GDK_PIXBUF_ROTATE_CLOCKWISE);
+            g_object_unref(q->pb);
+            q->pb = pb_rotate;
             snprintf(infotext, sizeof infotext, "(Rotated right)");
             swap(&q->orig_w, &q->orig_h);
             swap(&q->win_w, &q->win_h);
@@ -1039,11 +1047,15 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             correct_image_position(q);
             update_image(q, REDRAW);
             break;
+          }
 
             /* Rotate left */
 
           case 'l':
-            imlib_image_orientate(3);
+          {
+            GdkPixbuf* pb_rotate = gdk_pixbuf_rotate_simple(q->pb, GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
+            g_object_unref(q->pb);
+            q->pb = pb_rotate;
             snprintf(infotext, sizeof infotext, "(Rotated left)");
             swap(&q->orig_w, &q->orig_h);
             swap(&q->win_w, &q->win_h);
@@ -1051,6 +1063,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             correct_image_position(q);
             update_image(q, REDRAW);
             break;
+          }
 
             /* Center image on background */
 
@@ -1087,10 +1100,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             if (delay > 1000) {
               delay-=1000;
               snprintf(infotext, sizeof infotext, "(Slideshow-Delay: %d seconds (-1)", delay/1000);
-              update_image(q,MOVED);
+              update_image(q, MOVED);
             }else{
               snprintf(infotext, sizeof infotext, "(Slideshow-Delay: can not be less than 1 second!)");
-              update_image(q,MOVED);
+              update_image(q, MOVED);
             }
             break;
 
@@ -1137,10 +1150,21 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             }
             if((!fullscreen && do_grab) || (fullscreen && !disable_grab))
             {
-              gdk_keyboard_grab(q->win, FALSE, CurrentTime);
+#if GDK_MAJOR_VERSION < 3
               gdk_pointer_grab(q->win, FALSE,
                   GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
                   GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, NULL, CurrentTime);
+              gdk_keyboard_grab(q->win, FALSE, CurrentTime);
+#else
+              GdkDisplay* display = gdk_display_get_default();
+              GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
+              GdkDevice* pointer = gdk_device_manager_get_client_pointer(device_manager);
+              gdk_device_grab(pointer, q->win, GDK_OWNERSHIP_NONE, FALSE,
+                  GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
+                  GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, CurrentTime);
+              GdkDevice* keyboard = gdk_device_get_associated_device(pointer);
+              gdk_device_grab(keyboard, q->win, GDK_OWNERSHIP_NONE, FALSE, FALSE, NULL, CurrentTime);
+#endif
             }
             else
             {
@@ -1208,6 +1232,8 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             break;
         }
       }
+      break;
+    }
     default:
       break;
   }
