@@ -24,192 +24,109 @@ static struct timeval load_before, load_after;
 static double load_elapsed;
 static GdkCursor *cursor, *visible_cursor, *invisible_cursor;
 
-Imlib_Image im_from_pixbuf_loader(char * image_name, int * has_alpha, GError ** err)
+
+int img_from_pixbuf_loader(char * image_name, int * has_alpha, GError ** err, qiv_image *q)
 {
-  GError    *error=NULL;
-  GdkPixbuf *pixbuf_ori;
-  GdkPixbuf *pixbuf;
-  char * argbdata;
-  guchar * pixels;
-  guchar * pixels_ori;
-  int i,j,k,rs;
-  int pb_w, pb_h;
-  Imlib_Image * im=NULL;
-  const gchar *gdk_orientation = NULL;
+    GError    *error=NULL;
+    GdkPixbuf *pixbuf_ori = NULL;
+    GdkPixbuf *pixbuf = NULL;
+    guchar * pixeldata = NULL;
+    int pb_w, pb_h;
+    const gchar *gdk_orientation = NULL;
 #ifdef SUPPORT_LCMS
-  char *icc_profile;
-  cmsHPROFILE h_emb_profile;
-  cmsHTRANSFORM h_emb_transform;
+    char *icc_profile;
+    cmsHPROFILE h_emb_profile;
+    cmsHTRANSFORM h_emb_transform;
 #endif
-
-
-  pixbuf_ori = gdk_pixbuf_new_from_file(image_name, &error);
-  if (error != NULL)
-  {
-    *err=error;
-  }
-  else
-  {
-#if GDK_PIXBUF_MINOR >= 12
-  if(autorotate)
+    pixbuf_ori = gdk_pixbuf_new_from_file(image_name, &error);
+    if (error != NULL)
     {
-      gdk_orientation = gdk_pixbuf_get_option(pixbuf_ori, "orientation");
-      if(gdk_orientation)
-      {
+        *err=error;
+        q->pb = NULL;
+        return -1;
+    }
+    *has_alpha = gdk_pixbuf_get_has_alpha(pixbuf_ori);
+#if GDK_PIXBUF_MINOR >= 12
+    if(autorotate)
+    {
+        gdk_orientation = gdk_pixbuf_get_option(pixbuf_ori, "orientation");
+        if(gdk_orientation)
+        {
 #ifdef DEBUG
-        printf("orientation %s\n", gdk_orientation);
+            printf("orientation %s\n", gdk_orientation);
 #endif
-        pixbuf = gdk_pixbuf_apply_embedded_orientation (pixbuf_ori);
-        g_object_unref(pixbuf_ori);
-        pixbuf_ori = pixbuf;
-      }
+            pixbuf = gdk_pixbuf_apply_embedded_orientation(pixbuf_ori);
+            g_object_unref(pixbuf_ori);
+            pixbuf_ori = pixbuf;
+        }
     }
 #else
 #warning autoration needs at least gdk version 2.12
 #endif
 
-    *has_alpha =  ( gdk_pixbuf_get_n_channels(pixbuf_ori) == 4) ? 1 : 0;
-
-    pixels_ori = gdk_pixbuf_get_pixels(pixbuf_ori);
-    /* create checkboard if image has transparency */
-    if(*has_alpha)
-    {
-      pixbuf = gdk_pixbuf_composite_color_simple(pixbuf_ori, gdk_pixbuf_get_width(pixbuf_ori),
-                   gdk_pixbuf_get_height(pixbuf_ori), GDK_INTERP_NEAREST, 255, 0x08, 0x00666666, 0x00aaaaaa);
-
-      pixels = gdk_pixbuf_get_pixels(pixbuf);
-    }
-    else
-    {
-      pixbuf = pixbuf_ori;
-      pixels = pixels_ori;
-    }
-
-#ifdef DEBUG
-    printf("channels %i\n", gdk_pixbuf_get_n_channels(pixbuf));
-    printf("rowstride %i\n", gdk_pixbuf_get_rowstride(pixbuf));
-#endif
-
-    pb_w = gdk_pixbuf_get_width(pixbuf);
-    pb_h = gdk_pixbuf_get_height(pixbuf);
-
-    argbdata=malloc(4 * pb_w * pb_h);
-
-    /* create imlib2 compatible data */
-    if(*has_alpha)
-    {
-      for(i=0; i< pb_w*pb_h*4; i=i+4)
-      {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-        argbdata[i+0]=pixels[i+2]; // B
-        argbdata[i+1]=pixels[i+1]; // G
-        argbdata[i+2]=pixels[i];   // R
-        // keep old alpha values
-        argbdata[i+3]=pixels_ori[i+3]; // Alpha
-#else
-        // BIG_ENDIAN
-        argbdata[i+3]=pixels[i+2]; // B
-        argbdata[i+2]=pixels[i+1]; // G
-        argbdata[i+1]=pixels[i];   // R
-        // keep old alpha values
-        argbdata[i+0]=pixels_ori[i+3]; // Alpha
-#endif
-      }
-    }
-    else
-    {
-      rs = gdk_pixbuf_get_rowstride (pixbuf);
-      k=0;
-      for(i=0; i < pb_h; i++)
-      {
-        for(j=0; j< pb_w*3; j+=3)
-        {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-          argbdata[k++]=pixels[i*rs+j+2]; // B
-          argbdata[k++]=pixels[i*rs+j+1]; // G
-          argbdata[k++]=pixels[i*rs+j];   // R
-          argbdata[k++]=0;
-#else
-          argbdata[k++]=0;
-          argbdata[k++]=pixels[i*rs+j];   // R
-          argbdata[k++]=pixels[i*rs+j+1]; // G
-          argbdata[k++]=pixels[i*rs+j+2]; // B
-#endif
-        }
-      }
-    }
-
 #ifdef SUPPORT_LCMS
+    pixeldata = gdk_pixbuf_get_pixels(pixbuf_ori);
+    pb_w = gdk_pixbuf_get_width(pixbuf_ori);
+    pb_h = gdk_pixbuf_get_height(pixbuf_ori);
+
     if((icc_profile=get_icc_profile(image_name)))
     {
-      h_emb_profile = cmsOpenProfileFromMem(icc_profile+sizeof(cmsUInt32Number), *(cmsUInt32Number *)icc_profile);
-      if(h_display_profile==NULL)
-      {
-        h_display_profile = cmsCreate_sRGBProfile();
-      }
-
-      h_emb_transform = cmsCreateTransform(h_emb_profile,
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-          TYPE_BGRA_8,
-          h_display_profile,
-          TYPE_BGRA_8,
-#else
-          TYPE_ARGB_8,
-          h_display_profile,
-          TYPE_ARGB_8,
+#ifdef DEBUG
+        g_print("has color profile\n");
 #endif
-          INTENT_PERCEPTUAL, 0);
-      if(h_emb_transform)
-      {
-        cmsDoTransform(h_emb_transform, argbdata, argbdata, pb_w*pb_h);
-        cmsCloseProfile(h_emb_profile);
-        cmsDeleteTransform(h_emb_transform);
-      }
-      else
-      {
-        printf("qiv warning: %s contains corrupt color profile\n",image_name);
-      }
-      free(icc_profile);
+        h_emb_profile = cmsOpenProfileFromMem(icc_profile+sizeof(cmsUInt32Number), *(cmsUInt32Number *)icc_profile);
+        if(h_display_profile==NULL)
+        {
+            h_display_profile = cmsCreate_sRGBProfile();
+        }
+
+        if (*has_alpha == 0)
+        {
+            h_emb_transform = cmsCreateTransform(h_emb_profile, TYPE_RGB_8, h_display_profile, TYPE_RGB_8, INTENT_PERCEPTUAL, 0);
+        }
+        else
+        {
+            h_emb_transform = cmsCreateTransform(h_emb_profile, TYPE_RGBA_8, h_display_profile, TYPE_RGBA_8, INTENT_PERCEPTUAL, 0);
+        }
+        if(h_emb_transform)
+        {
+            cmsDoTransform(h_emb_transform, pixeldata, pixeldata, pb_w*pb_h);
+            cmsCloseProfile(h_emb_profile);
+            cmsDeleteTransform(h_emb_transform);
+        }
+        else
+        {
+            printf("qiv warning: %s contains corrupt color profile\n",image_name);
+        }
+        free(icc_profile);
     }
 
     /* do the color transform */
     else if (cms_transform && h_cms_transform)
     {
-      cmsDoTransform(h_cms_transform, argbdata, argbdata, pb_w*pb_h);
+        cmsDoTransform(h_cms_transform, pixeldata, pixeldata, pb_w*pb_h);
     }
 #endif
-    im = imlib_create_image_using_copied_data(pb_w, pb_h, (DATA32*)argbdata);
-    free(argbdata);
-    if(*has_alpha)
-    {
-      g_object_unref(pixbuf);
-    }
-    g_object_unref(pixbuf_ori);
 
 #ifdef HAVE_EXIF
     artist = get_exif_artist(image_name);
     //printf("DEBUG: Artist: %s\n", artist);
 #endif
-  }
-  return im;
+    q->pb = gdk_pixbuf_copy(pixbuf_ori);
+    g_object_unref(pixbuf_ori);
+    return 0;
 }
 
 // small wrapper around pixbuf and imlib loader
-Imlib_Image load_image(char* image_name, int* has_alpha)
+int load_image(char* image_name, int* has_alpha, qiv_image *q)
 {
-  Imlib_Image * im = NULL;
+  //Imlib_Image * im = NULL;
+  int im;
   GError * error = NULL;
 
-  // try gdk_pixbuf for loading first
-  im = im_from_pixbuf_loader(image_name, has_alpha, &error);
+  im = img_from_pixbuf_loader(image_name, has_alpha, &error, q);
 
-  if(!im)
-  {
-    // imlib as fallback
-    im = imlib_load_image(image_name);
-  }
-
-  if(!im && error != NULL)
+  if(im != 0 && error != NULL)
   {
     // Report GDK error to user if imlib failed, too
     fprintf(stderr, "Unable to read file: %s\n", error->message);
@@ -229,15 +146,13 @@ void qiv_load_image(qiv_image *q)
 {
   struct stat statbuf;
   const char * image_name = image_names[ image_idx];
-  Imlib_Image * im=NULL;
+  //Imlib_Image * im=NULL;
+  int im=0;
   int has_alpha=0, rot;
 
   q->exposed=0;
   gettimeofday(&load_before, 0);
 
-  if (imlib_context_get_image())
-    imlib_free_image();
-
   stat(image_name, &statbuf);
   current_mtime = statbuf.st_mtime;
   file_size = statbuf.st_size;
@@ -246,20 +161,16 @@ void qiv_load_image(qiv_image *q)
   g_print("loading %s\n",image_name);
 #endif
 
-  im = load_image((char*)image_name, &has_alpha);
+  im = load_image((char*)image_name, &has_alpha, q);
 
-  if (!im) { /* error */
+  if (im) { /* error */
     q->error = 1;
     q->orig_w = 400;
     q->orig_h = 300;
   } else { /* Retrieve image properties */
-    imlib_context_set_image(im);
-    if(has_alpha) {
-      imlib_image_set_has_alpha(has_alpha);
-    }
     q->error = 0;
-    q->orig_w = imlib_image_get_width();
-    q->orig_h = imlib_image_get_height();
+    q->orig_w = gdk_pixbuf_get_width(q->pb);
+    q->orig_h = gdk_pixbuf_get_height(q->pb);
 
     if (rotation > 10) {
       /* conditional rotation -- apply rotation only if image fits better */
@@ -274,7 +185,6 @@ void qiv_load_image(qiv_image *q)
       rot = rotation;
 
     if (rot) {
-      imlib_image_orientate(rot);
       if (rot != 2) {
         swap(&q->orig_w, &q->orig_h);
         swap(&q->win_w, &q->win_h);
@@ -302,7 +212,9 @@ void qiv_load_image(qiv_image *q)
     qiv_exit(0);
   }
 
-  gdk_window_set_background(q->win, im ? &image_bg : &error_bg);
+  // This does not seem to be needed
+  /*gdk_window_set_background(q->win, im ? &image_bg : &error_bg);*/
+  /*printf("set background\n");*/
 
   gettimeofday(&load_after, 0);
   load_elapsed = ((load_after.tv_sec +  load_after.tv_usec / 1.0e6) -
@@ -315,45 +227,13 @@ void qiv_load_image(qiv_image *q)
 //    }
 }
 
-static void setup_imlib_for_drawable(GdkDrawable * d)
-{
-  imlib_context_set_dither(1); /* dither for depths < 24bpp */
-  imlib_context_set_display(
-    gdk_x11_drawable_get_xdisplay(d));
-  imlib_context_set_visual(
-    gdk_x11_visual_get_xvisual(gdk_drawable_get_visual(d)));
-  imlib_context_set_colormap(
-    gdk_x11_colormap_get_xcolormap(gdk_drawable_get_colormap(d)));
-  imlib_context_set_drawable(
-    gdk_x11_drawable_get_xid(d));
-}
-
-static void setup_imlib_color_modifier(qiv_color_modifier q)
-{
-  if (q.gamma == DEFAULT_GAMMA &&
-      q.brightness == DEFAULT_BRIGHTNESS &&
-      q.contrast == DEFAULT_CONTRAST) {
-    if (imlib_context_get_color_modifier())
-      imlib_free_color_modifier();
-    return;
-  }
-
-  if (imlib_context_get_color_modifier())
-    imlib_reset_color_modifier();
-  else
-    imlib_context_set_color_modifier(imlib_create_color_modifier());
-
-  imlib_modify_color_modifier_gamma(q.gamma / 256.0);
-  imlib_modify_color_modifier_brightness((q.brightness - 256) / 256.0);
-  imlib_modify_color_modifier_contrast(q.contrast / 256.0);
-}
-
 static void setup_win(qiv_image *q)
 {
   GdkWindowAttr attr;
   static GList  icon_list = {NULL, NULL, NULL};
 
-  destroy_image(q);
+  /*TODO: thw 25.5.24: should that even be here?*/
+  //destroy_image(q);
 
   if (!fullscreen) {
     attr.window_type=GDK_WINDOW_TOPLEVEL;
@@ -417,17 +297,14 @@ static void setup_win(qiv_image *q)
   if(icon_list.data==NULL)
   {
     icon_list.data = gdk_pixbuf_new_from_xpm_data(qiv_icon);
-    invisible_cursor = gdk_cursor_new(GDK_BLANK_CURSOR);
-    cursor = visible_cursor = gdk_cursor_new(CURSOR);
+    invisible_cursor = gdk_cursor_new_for_display(gdk_display_get_default(), GDK_BLANK_CURSOR);
+    cursor = visible_cursor = gdk_cursor_new_for_display(gdk_display_get_default(), CURSOR);
   }
 
   gdk_window_set_icon_list(q->win, &icon_list);
   gdk_window_set_cursor(q->win, cursor);
 
-  q->bg_gc = gdk_gc_new(q->win);
-  gdk_gc_set_foreground(q->bg_gc, &image_bg);
-
-  setup_imlib_for_drawable(GDK_DRAWABLE(q->win));
+  //setup_imlib_for_drawable(GDK_DRAWABLE(q->win));
 }
 
 void hide_cursor(qiv_image *q)
@@ -447,15 +324,13 @@ void show_cursor(qiv_image *q)
 void set_desktop_image(qiv_image *q)
 {
   GdkWindow *root_win = gdk_get_default_root_window();
-  GdkVisual *gvis = gdk_drawable_get_visual(root_win);
-  GdkPixmap *temp;
-  gchar     *buffer;
+#if GDK_MAJOR_VERSION < 3
+  GdkPixmap *pixmap;
+  GdkBitmap *mask = NULL;
 
   gint root_w = screen_x, root_h = screen_y;
   gint root_x = 0, root_y = 0;
 
-  Pixmap x_pixmap, x_mask;
-
   if (to_root || to_root_t) {
     root_w = q->win_w;
     root_h = q->win_h;
@@ -466,40 +341,39 @@ void set_desktop_image(qiv_image *q)
     root_y = (screen_y - root_h) / 2;
   }
 
-  setup_imlib_for_drawable(GDK_DRAWABLE(root_win));
-
-  imlib_render_pixmaps_for_whole_image_at_size(&x_pixmap, &x_mask, root_w, root_h);
 #ifdef DEBUG
-  if (x_mask)  g_print("*** image has transparency\n");
+  if (mask)  g_print("*** image has transparency\n");
 #endif
 
-  if(x_pixmap) {
-    GdkPixmap * p = gdk_pixmap_foreign_new(x_pixmap);
-    gdk_drawable_set_colormap(GDK_DRAWABLE(p),
-			      gdk_drawable_get_colormap(GDK_DRAWABLE(root_win)));
-    if (to_root_t) {
-      gdk_window_set_back_pixmap(root_win, p, FALSE);
+  gdk_window_clear(root_win);
+  if (to_root_t) {
+    gdk_pixbuf_render_pixmap_and_mask_for_colormap (q->pb, gdk_window_get_colormap(q->win), &pixmap, &mask, 128);
+    gdk_window_set_back_pixmap(root_win, pixmap, FALSE);
+    printf("to root_t\n");
+  } else {
+    printf("to root x %d y %d w %d h %d\n", root_x, root_y, root_w, root_h);
+
+    GdkPixbuf* pb_tmp;
+    if (to_root_s){
+      if (gdk_pixbuf_get_has_alpha(q->pb)){
+          pb_tmp = gdk_pixbuf_scale_simple(q->pb, screen_x, screen_y, GDK_INTERP_BILINEAR);
+      } else {
+          pb_tmp = gdk_pixbuf_composite_color_simple(q->pb, screen_x, screen_y, GDK_INTERP_BILINEAR,
+                255, 0x08, 0x00666666, 0x00aaaaaa);
+      }
     } else {
-      GdkGC *rootGC;
-      buffer = xcalloc(1, screen_x * screen_y * gdk_visual_get_depth(gvis) / 8);
-      rootGC = gdk_gc_new(root_win);
-      temp = gdk_pixmap_create_from_data(root_win, buffer, screen_x,
-                                         screen_y, gdk_visual_get_depth(gvis), &image_bg, &image_bg);
-     gdk_drawable_set_colormap(GDK_DRAWABLE(temp),
-			      gdk_drawable_get_colormap(GDK_DRAWABLE(root_win)));
-      gdk_draw_drawable(temp, rootGC, p, 0, 0, root_x, root_y, root_w, root_h);
-      gdk_window_set_back_pixmap(root_win, temp, FALSE);
-      g_object_unref(temp);
-      g_object_unref(rootGC);
-      free(buffer);
+        pb_tmp = gdk_pixbuf_new(GDK_COLORSPACE_RGB, gdk_pixbuf_get_has_alpha(q->pb), 8, screen_x, screen_y);
+        gdk_pixbuf_copy_area(q->pb, 0, 0, q->win_w, q->win_h, pb_tmp, root_x, root_y);
     }
-    g_object_unref(p);
-    imlib_free_pixmap_and_mask(x_pixmap);
+
+    gdk_pixbuf_render_pixmap_and_mask_for_colormap (pb_tmp, gdk_window_get_colormap(q->win), &pixmap, &mask, 128);
+    gdk_window_set_back_pixmap(root_win, pixmap, FALSE);
+    g_object_unref(pb_tmp);
   }
   gdk_window_clear(root_win);
-  gdk_flush();
+  gdk_display_flush(gdk_display_get_default());
+#endif
 
-  setup_imlib_for_drawable(q->win);
 }
 
 void zoom_in(qiv_image *q)
@@ -598,36 +472,29 @@ void zoom_maxpect(qiv_image *q)
 
 void reload_image(qiv_image *q)
 {
-  Imlib_Image *im;
+  int im;
   int has_alpha = 0;
 
-  imlib_image_set_changes_on_disk();
+  //imlib_image_set_changes_on_disk();
 
-  im = load_image(image_names[image_idx], &has_alpha);
+  im = load_image(image_names[image_idx], &has_alpha, q);
 
-  if (!im && watch_file)
+  if (im && watch_file)
     return;
 
   struct stat statbuf;
   stat(image_names[image_idx], &statbuf);
   current_mtime = statbuf.st_mtime;
 
-  if (imlib_context_get_image())
-    imlib_free_image();
-
-  if (!im)
+  if (im!=0)
   {
     q->error = 1;
     q->orig_w = 400;
     q->orig_h = 300;
   } else { /* Retrieve image properties */
     q->error = 0;
-    imlib_context_set_image(im);
-    if(has_alpha) {
-      imlib_image_set_has_alpha(has_alpha);
-    }
-    q->orig_w = imlib_image_get_width();
-    q->orig_h = imlib_image_get_height();
+    q->orig_w = gdk_pixbuf_get_width(q->pb);
+    q->orig_h = gdk_pixbuf_get_height(q->pb);
   }
 
   q->win_w = (gint)(q->orig_w * (1 + zoom_factor * 0.1) * highDPIfactor);
@@ -677,17 +544,19 @@ void reset_coords(qiv_image *q)
 
 void update_image(qiv_image *q, int mode)
 {
-  static GdkPixmap * m = NULL;
-  Pixmap x_pixmap, x_mask;
+  //static GdkPixmap * m = NULL;
+  //Pixmap x_pixmap, x_mask;
+#if GDK_MAJOR_VERSION < 3
+  GdkBitmap *mask = NULL;
+#endif
   double elapsed=0;
-  struct timeval before, after;
   int i;
   cairo_t *cr;
 
   if (q->error) {
     g_snprintf(q->win_title, sizeof q->win_title,
         "qiv: ERROR! cannot load image: %s", image_names[image_idx]);
-    gdk_beep();
+    gdk_display_beep(gdk_display_get_default());
 
     /* take this image out of the file list */
     --images;
@@ -711,25 +580,8 @@ void update_image(qiv_image *q, int mode)
     return;
 
   } else {
-    if (mode == REDRAW || mode == FULL_REDRAW)
-      setup_imlib_color_modifier(q->mod);
 
     if (mode == MOVED) {
-      if (transparency && used_masks_before) {
-        /* there should be a faster way to update the mask, but how? */
-        if (q->p)
-        {
-          imlib_free_pixmap_and_mask(GDK_PIXMAP_XID(q->p));
-          g_object_unref(q->p);
-        }
-        if (m) g_object_unref(m);
-	imlib_render_pixmaps_for_whole_image_at_size(&x_pixmap, &x_mask, q->win_w, q->win_h);
-	q->p = gdk_pixmap_foreign_new(x_pixmap);
-	gdk_drawable_set_colormap(GDK_DRAWABLE(q->p),
-				  gdk_drawable_get_colormap(GDK_DRAWABLE(q->win)));
-	m = gdk_pixmap_foreign_new(x_mask);
-     }
-
       g_snprintf(q->win_title, sizeof q->win_title,
                  "qiv: %s (%dx%d) %d%% [%d/%d] b%d/c%d/g%d %s",
                  image_names[image_idx], q->orig_w, q->orig_h,
@@ -741,39 +593,8 @@ void update_image(qiv_image *q, int mode)
     } // mode == MOVED
     else
     {
-      if(mode != MIN_REDRAW)
-      {
-        GdkPixmap * pix_ptr = NULL;
-        if (q->p) {
-          imlib_free_pixmap_and_mask(GDK_PIXMAP_XID(q->p));
-          g_object_unref(q->p);
-        }
-        if (m) g_object_unref(m);
-
-        /* calculate elapsed time while we render image */
-        gettimeofday(&before, 0);
-        imlib_render_pixmaps_for_whole_image_at_size(&x_pixmap, &x_mask, q->win_w, q->win_h);
-        gettimeofday(&after, 0);
-        elapsed = ((after.tv_sec +  after.tv_usec / 1.0e6) -
-            (before.tv_sec + before.tv_usec / 1.0e6));
-
-        pix_ptr = gdk_pixmap_lookup (x_pixmap);
-        if(pix_ptr == NULL)
-        {
-          q->p = gdk_pixmap_foreign_new_for_screen(screen, x_pixmap, q->win_w, q->win_h, 24);
-        }
-        else
-        {
-          q->p = pix_ptr;
-          g_object_ref(q->p);
-        }
-        gdk_drawable_set_colormap(GDK_DRAWABLE(q->p),
-            gdk_drawable_get_colormap(GDK_DRAWABLE(q->win)));
-        m = x_mask == None ? NULL : gdk_pixmap_foreign_new_for_screen(screen, x_mask, q->win_w, q->win_h, 1);
-      }
-
 #ifdef DEBUG
-      if (m)  g_print("*** image has transparency\n");
+      if (gdk_pixbuf_get_has_alpha(q->pb))  g_print("*** image has transparency\n");
 #endif
 
       g_snprintf(q->win_title, sizeof q->win_title,
@@ -803,6 +624,9 @@ void update_image(qiv_image *q, int mode)
   }
 
   if (!fullscreen) {
+#if GDK_MAJOR_VERSION < 3
+    gdk_window_clear(q->win);
+#endif
     GdkGeometry geometry = {
       .min_width = q->win_w,
       .min_height = q->win_h,
@@ -821,25 +645,83 @@ void update_image(qiv_image *q, int mode)
 
     if(mode != MIN_REDRAW)
        gdk_window_move_resize(q->win, q->win_x, q->win_y, q->win_w, q->win_h);
-
     if (!q->error) {
-      gdk_window_set_back_pixmap(q->win, q->p, FALSE);
+      //gdk_window_set_back_pixmap(q->win, q->p, FALSE);
+      cr = gdk_cairo_create(q->win);
+
+      GdkPixbuf* pb_tmp;
+      if (gdk_pixbuf_get_n_channels(q->pb) == 3){
+          pb_tmp = gdk_pixbuf_scale_simple(q->pb, q->win_w, q->win_h, GDK_INTERP_BILINEAR);
+      } else {
+          GdkPixbuf* pb_tmp2;
+          pb_tmp2 = gdk_pixbuf_scale_simple(q->pb, q->win_w, q->win_h, GDK_INTERP_BILINEAR);
+#if GDK_MAJOR_VERSION < 3
+          gdk_pixbuf_render_pixmap_and_mask_for_colormap (pb_tmp2, gdk_window_get_colormap(q->win), NULL, &mask, 128);
+#endif
+          pb_tmp = gdk_pixbuf_composite_color_simple(q->pb, q->win_w, q->win_h, GDK_INTERP_BILINEAR,
+                  255, 0x08, 0x00666666, 0x00aaaaaa);
+          g_object_unref(pb_tmp2);
+      }
+      gdk_cairo_set_source_pixbuf (cr, pb_tmp, 0, 0);
+      //cairo_rectangle(cr, 0, 0, q->win_w, q->win_h);
+      cairo_paint(cr);
+      cairo_destroy(cr);
       /* remove or set transparency mask */
       if (used_masks_before) {
-        if (transparency)
-          gdk_window_shape_combine_mask(q->win, m, 0, 0);
+        if (transparency) {
+#if GDK_MAJOR_VERSION < 3
+          gdk_window_shape_combine_mask(q->win, mask, 0, 0);
+#else
+          // GDK3 Code
+          cairo_surface_t *surface;
+          cairo_t *cr_m;
+          cairo_region_t *region;
+
+          surface = cairo_image_surface_create (CAIRO_FORMAT_A1, q->win_w, q->win_h);
+          cr_m = cairo_create(surface);
+          gdk_cairo_set_source_pixbuf (cr_m, q->pb, 0, 0);
+          cairo_paint(cr_m);
+          region = gdk_cairo_region_create_from_surface (surface);
+          gdk_window_shape_combine_region(q->win, region, 0, 0);
+          cairo_destroy(cr_m);
+          printf("combine shape\n");
+#endif
+        }
         else
+        {
+#if GDK_MAJOR_VERSION < 3
           gdk_window_shape_combine_mask(q->win, 0, 0, 0);
+#endif
+          used_masks_before = 0;
+        }
       }
       else
       {
-        if (transparency && m) {
-          gdk_window_shape_combine_mask(q->win, m, 0, 0);
+        if (transparency && (gdk_pixbuf_get_n_channels(q->pb) == 4)) {
+#if GDK_MAJOR_VERSION < 3
+          gdk_window_shape_combine_mask(q->win, mask, 0, 0);
+#else
+          // GDK3 Code
+          cairo_surface_t *surface;
+          cairo_t *cr_m;
+          cairo_region_t *region;
+
+          surface = cairo_image_surface_create (CAIRO_FORMAT_A1, q->win_w, q->win_h);
+          cr_m = cairo_create(surface);
+          gdk_cairo_set_source_pixbuf (cr_m, q->pb, 0, 0);
+          cairo_paint(cr_m);
+          cairo_destroy(cr_m);
+          region = gdk_cairo_region_create_from_surface (surface);
+          gdk_window_shape_combine_region(q->win, region, 0, 0);
+          printf("combine shape\n");
+#endif
           used_masks_before=1;
         }
       }
+      g_object_unref(pb_tmp);
     }
-    gdk_window_clear(q->win);
+    //printf("----\n");
+    //gdk_window_clear(q->win);
 
     if(statusbar_window)
     {
@@ -899,7 +781,16 @@ void update_image(qiv_image *q, int mode)
 # define statusbar_x monitor[q->mon_id].width
 # define statusbar_y monitor[q->mon_id].height
     if (mode == FULL_REDRAW) {
+#if GDK_MAJOR_VERSION < 3
       gdk_window_clear(q->win);
+#else
+      cr = gdk_cairo_create (q->win);
+      set_cairo_color(cr, 0);
+      cairo_rectangle(cr, 0, 0, monitor[q->mon_id].width, monitor[q->mon_id].height);
+      cairo_fill (cr);
+      cairo_destroy (cr);
+#endif
+
     } else {
       cr = gdk_cairo_create (q->win);
       set_cairo_color(cr, 0);
@@ -929,24 +820,53 @@ void update_image(qiv_image *q, int mode)
       cairo_destroy(cr);
     }
 
-    /* remove or set transparency mask */
-    if (used_masks_before) {
-      if (transparency)
-        gdk_window_shape_combine_mask(q->win, m, q->win_x, q->win_y);
-      else
-        gdk_window_shape_combine_mask(q->win, 0, q->win_x, q->win_y);
-    }
-    else
-    {
-      if (transparency && m) {
-        gdk_window_shape_combine_mask(q->win, m, q->win_x, q->win_y);
-        used_masks_before=1;
-      }
-    }
+    if (!q->error){
+       cr = gdk_cairo_create (q->win);
 
-    if (!q->error)
-      gdk_draw_drawable(q->win, q->bg_gc, q->p, 0, 0,
-                        q->win_x, q->win_y, q->win_w, q->win_h);
+       GdkPixbuf* pb_tmp;
+       if (gdk_pixbuf_get_n_channels(q->pb) == 3){
+           pb_tmp = gdk_pixbuf_scale_simple(q->pb, q->win_w, q->win_h, GDK_INTERP_BILINEAR);
+       } else {
+           GdkPixbuf* pb_tmp2;
+           pb_tmp2 = gdk_pixbuf_scale_simple(q->pb, q->win_w, q->win_h, GDK_INTERP_BILINEAR);
+#if GDK_MAJOR_VERSION < 3
+           gdk_pixbuf_render_pixmap_and_mask_for_colormap (pb_tmp2, gdk_window_get_colormap(q->win), NULL, &mask, 128);
+#endif
+           pb_tmp  = gdk_pixbuf_composite_color_simple(pb_tmp2, q->win_w, q->win_h, GDK_INTERP_BILINEAR,
+                      255, 0x08, 0x00666666, 0x00aaaaaa);
+           g_object_unref(pb_tmp2);
+       }
+       gdk_cairo_set_source_pixbuf (cr, pb_tmp, q->win_x, q->win_y);
+       g_object_unref(pb_tmp);
+       /* remove or set transparency mask */
+       if (used_masks_before) {
+         if (transparency) {
+#if GDK_MAJOR_VERSION < 3
+           gdk_window_shape_combine_mask(q->win, mask, q->win_x, q->win_y);
+#endif
+         }
+         else
+         {
+#if GDK_MAJOR_VERSION < 3
+           gdk_window_shape_combine_mask(q->win, 0, q->win_x, q->win_y);
+#endif
+           used_masks_before = 0;
+         }
+       }
+       else
+       {
+         if (transparency && (gdk_pixbuf_get_n_channels(q->pb) == 4)) {
+#if GDK_MAJOR_VERSION < 3
+           gdk_window_shape_combine_mask(q->win, mask, q->win_x, q->win_y);
+#endif
+           used_masks_before=1;
+         }
+       }
+
+       cairo_rectangle(cr, 0, 0, q->win_w, q->win_h);
+       cairo_paint(cr);
+       cairo_destroy (cr);
+    }
 
     if (statusbar_fullscreen) {
       cr = gdk_cairo_create (q->win);
@@ -1016,7 +936,10 @@ void update_image(qiv_image *q, int mode)
     gdk_window_move_resize(q->win, monitor[q->mon_id].x, monitor[q->mon_id].y,
         monitor[q->mon_id].width, monitor[q->mon_id].height);
   }
-  gdk_flush();
+  gdk_display_flush(gdk_display_get_default());
+#if GDK_MAJOR_VERSION < 3
+  if(mask) g_object_unref(mask);
+#endif
 }
 
 
@@ -1029,16 +952,14 @@ void reset_mod(qiv_image *q)
 
 void destroy_image(qiv_image *q)
 {
-  if (q->p) {
-    imlib_free_pixmap_and_mask(GDK_PIXMAP_XID(q->p));
-    g_object_unref(q->p);
-  }
+  //printf("destroy image\n");
   if (q->win) g_object_unref(q->win);
-  if (q->bg_gc) g_object_unref(q->bg_gc);
 
-  q->p=NULL;
-  q->win=NULL;
-  q->bg_gc=NULL;
+  q->win= NULL;
+  if(!first) {
+     if (q->pb) g_object_unref(q->pb);
+     q->pb = NULL;
+  }
 }
 
 void setup_magnify(qiv_image *q, qiv_mgl *m)
@@ -1068,7 +989,7 @@ void setup_magnify(qiv_image *q, qiv_mgl *m)
    gdk_window_set_geometry_hints(m->win, &mgl_hints,
      GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
    gdk_window_set_decorations(m->win, GDK_DECOR_BORDER);
-   gdk_flush();
+   gdk_display_flush(gdk_display_get_default());
 }
 
 void update_magnify(qiv_image *q, qiv_mgl *m, int mode, gint xcur, gint ycur)
@@ -1076,6 +997,7 @@ void update_magnify(qiv_image *q, qiv_mgl *m, int mode, gint xcur, gint ycur)
 //   GdkWindowAttr mgl_attr;
 //   GdkGeometry mgl_hints;
   register  gint xx, yy;  // win_pos_x, win_pos_y;
+  cairo_t *cr;
 
 /***********
   if (mode == FULL_REDRAW) {
@@ -1115,12 +1037,18 @@ void update_magnify(qiv_image *q, qiv_mgl *m, int mode, gint xcur, gint ycur)
     if(yy + m->win_h / m->zoom > q->orig_h) {
       yy=q->orig_h - m->win_h / m->zoom;
     }
+    if (yy < 0){
+        yy=0;
+    }
+    if (xx < 0){
+        xx=0;
+    }
 
-
-    setup_imlib_for_drawable(m->win);
-    imlib_render_image_part_on_drawable_at_size(xx, yy, m->win_w / m->zoom , m->win_h / m->zoom,
-						0, 0, m->win_w, m->win_h);
-    setup_imlib_for_drawable(q->win);
+    cr = gdk_cairo_create(m->win);
+    cairo_scale(cr, m->zoom, m->zoom);
+    gdk_cairo_set_source_pixbuf(cr, q->pb, -xx, -yy);
+    cairo_paint(cr);
+    cairo_destroy(cr);
     gdk_window_show(m->win);
 
     xx= m->frame_x + xcur - 50 - m->win_w;
@@ -1140,7 +1068,7 @@ void update_magnify(qiv_image *q, qiv_mgl *m, int mode, gint xcur, gint ycur)
 //    printf("MGL: m->frame_x: %d, m->frame_y: %d, xx: %d, yy: %d\n", m->frame_x, m->frame_y, xx, yy);
     gdk_window_move(m->win, xx, yy);
   }
-  gdk_flush();
+  gdk_display_flush(gdk_display_get_default());
 }
 
 void center_image(qiv_image *q)
