@@ -7,6 +7,7 @@
   Policy       : GNU GPL
 */
 
+#define _DEFAULT_SOURCE
 #include <gdk/gdkx.h>
 #include <stdio.h>
 #include <signal.h>
@@ -25,8 +26,7 @@ qiv_image main_img;
 qiv_mgl   magnify_img; /* [lc] */
 
 static int check_extension(const char *);
-static void qiv_signal_usr1();
-static void qiv_signal_usr2();
+static void qiv_signal_usr(int sig);
 static gboolean qiv_handle_timer(gpointer);
 static void qiv_timer_restart(gpointer);
 
@@ -121,7 +121,6 @@ int main(int argc, char **argv)
   /* Load things from GDK/Imlib */
 
   qiv_main_loop = g_main_loop_new(NULL, TRUE);
-  cmap = gdk_colormap_get_system();
   screen_x = gdk_screen_width();
   screen_y = gdk_screen_height();
 
@@ -138,7 +137,7 @@ int main(int argc, char **argv)
     main_img.mon_id = user_screen;
   }
 
- /* statusbar with pango */
+  /* statusbar with pango */
   layout = pango_layout_new(gdk_pango_context_get());
   fontdesc = pango_font_description_from_string (STATUSBAR_FONT);
 
@@ -182,11 +181,6 @@ int main(int argc, char **argv)
 
   max_rand_num = images;
 
-  /* get colors */
-
-  color_alloc(ERROR_BG, &error_bg);
-  color_alloc(image_bg_spec, &image_bg);
-
   /* Display first image first, except in random mode */
 
   if (random_order)
@@ -207,8 +201,8 @@ int main(int argc, char **argv)
 
   signal(SIGTERM, finish);
   signal(SIGINT, finish);
-  signal(SIGUSR1, qiv_signal_usr1);
-  signal(SIGUSR2, qiv_signal_usr2);
+  signal(SIGUSR1, qiv_signal_usr);
+  signal(SIGUSR2, qiv_signal_usr);
 
   /* check for DPMS capability and disable it if slideshow
      was started from command options */
@@ -233,7 +227,6 @@ int main(int argc, char **argv)
 
 void qiv_exit(int code)
 {
-  if (cmap) gdk_colormap_unref(cmap);
   destroy_image(&main_img);
   dpms_enable();
 
@@ -247,18 +240,16 @@ void qiv_exit(int code)
 
 
 /*
- * functions for handling signals
+ * function for handling signals (only USR1 and USR2 handled)
  */
 
-static void qiv_signal_usr1()
+static void qiv_signal_usr(int sig)
 {
-  next_image(1);
-  qiv_load_image(&main_img);
-}
-
-static void qiv_signal_usr2()
-{
-  next_image(-1);
+  if (sig == SIGUSR1) {
+    next_image(1);
+  } else {
+    next_image(-1);
+  }
   qiv_load_image(&main_img);
 }
 
@@ -277,7 +268,7 @@ static gboolean qiv_handle_timer(gpointer data)
   if (*(int *)data || slide) {
     next_image(0);
     /* disable screensaver during slideshow */
-    XResetScreenSaver(GDK_DISPLAY());
+    XResetScreenSaver(gdk_x11_display_get_xdisplay(gdk_display_get_default()));
     qiv_load_image(&main_img);
   }
   return FALSE;
