@@ -9,10 +9,7 @@ gint            num_monitors;
 GdkScreen       *screen;
 GdkRectangle    *monitor;
 
-GdkColormap	*cmap; /* global colormap */
 char		*image_bg_spec = IMAGE_BG;
-GdkColor	image_bg; /* default background */
-GdkColor	error_bg; /* for the error window/screen */
 int		images;	/* Number of images in current collection */
 char		**image_names = NULL; /* Filenames of the images */
 int		image_idx = 0; /* Index of current image displayed. 0 = 1st image */
