@@ -7,6 +7,7 @@
   Original     : http://www.klografx.net/qiv/
 */
 
+#define _DEFAULT_SOURCE
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -480,7 +481,7 @@ int checked_atoi (const char *s)
 
     if (endptr == s || *endptr != '\0') {
 	g_print("Error: %s is not a valid number.\n", s);
-	gdk_exit(1);
+	exit(1);
     }
 
     return num;
@@ -493,7 +494,7 @@ float checked_atof (const char *s)
 
     if (endptr == s || *endptr != '\0') {
 	g_print("Error: %s is not a valid floating point number.\n", s);
-	gdk_exit(1);
+	exit(1);
     }
 
     return num;
@@ -506,7 +507,7 @@ void usage(char *name, int exit_status)
 	"See 'man qiv' or type 'qiv --help' for options.\n",
         VERSION);
 
-    gdk_exit(exit_status);
+    exit(exit_status);
 }
 
 void show_help(char *name, int exit_status)
@@ -594,7 +595,7 @@ void show_help(char *name, int exit_status)
     g_print("Homepage: http://qiv.spiegl.de/\n"
 	    "Please mail bug reports and comments to Andy Spiegl <qiv.andy@spiegl.de>\n");
 
-    gdk_exit(exit_status);
+    exit(exit_status);
 }
 
 /* returns a random number from the integers 0..num-1, either with
@@ -723,6 +724,7 @@ int rreadfile(const char *filename)
     return images - before_count;
 }
 
+/*
 gboolean color_alloc(const char *name, GdkColor *color)
 {
     gboolean result;
@@ -743,6 +745,7 @@ gboolean color_alloc(const char *name, GdkColor *color)
 
     return result;
 }
+*/
 
 void swap(int *a, int *b)
 {
@@ -792,6 +795,10 @@ int find_image(int images, char **image_names, char *name)
 }
 
 #ifdef SUPPORT_LCMS
+#ifndef JPEG_APP2
+#define JPEG_APP2 (JPEG_APP0 + 2)
+#endif
+
 char *get_icc_profile(char *filename)
 {
   FILE *infile;
@@ -812,11 +819,13 @@ char *get_icc_profile(char *filename)
    * "ICC_PROFILE\0"
    * Marker ID
    * No of total markers
-   * data length
    * data
    */
   const char icc_string[]="ICC_PROFILE";
-  int seq_max;
+  const size_t icc_string_len = 12; // len includes terminating \0
+  size_t seq_id, seq_max;
+  const size_t icc_offset = 14; // len + 2 bytes for id + max_id
+  int seq_size;
 
   /* Tiff ICC header:
    * Bytes
@@ -856,8 +865,8 @@ char *get_icc_profile(char *filename)
 
     jpeg_create_decompress(&cinfo);
     jpeg_stdio_src(&cinfo, infile);
-    jpeg_save_markers(&cinfo, 0xE2, 0xFFFF);
-    jpeg_save_markers(&cinfo, JPEG_COM, 0xFFFF);
+    jpeg_save_markers(&cinfo, JPEG_APP2, 0xFFFF); // ICC profile
+    jpeg_save_markers(&cinfo, JPEG_COM, 0xFFFF);  // JPEG comment
     jpg_ok = jpeg_read_header(&cinfo, 0);
     if (jpg_ok == 0) {
       qiv_exit(2);
@@ -871,17 +880,37 @@ char *get_icc_profile(char *filename)
       {
         if(i==0)
         {
-          seq_max=marker->data[13];
+          seq_max = marker->data[icc_string_len+1];
           tag_length = calloc(seq_max, sizeof(short));
           tag_ptr    = calloc(seq_max, sizeof(char *));
         }
-        // hmm, in theory both should be the same (tw)
-        tag_length[marker->data[12]-1] = marker->data_length - 14;
-//      tag_length[marker->data[12]-1] = (marker->data[16] << 8) +  marker->data[17];
+        seq_id = marker->data[icc_string_len];
+        seq_size = marker->data_length - icc_offset;
 
-        (tag_ptr[marker->data[12]-1])  = marker->data;
-        (tag_ptr[marker->data[12]-1]) += 14;
+        // workaround: if there is only one ICC header, ignore a wrong value by setting it to 1
+        if((seq_max == 1) && (seq_id != 1)){
+            printf("qiv warning: fixing incorrect ICC chunk ID in %s\n", filename);
+            seq_id = 1;
+        }
 
+        // Sanity checks
+        // id runs from 1 to max ID, max ID must stay same for all ICC tags, size of ICC data > 0
+        if((seq_id < 1) || (seq_id > seq_max) || (seq_size <= 0) || (seq_max != marker->data[icc_string_len+1])){
+          printf("qiv warning: %s contains illegal ICC tag\n", filename);
+          free(tag_length);
+          free(tag_ptr);
+          return NULL;
+        }
+        // we already had this ID
+        if(tag_length[seq_id-1]){
+          printf("qiv warning: %s contains duplicate ICC sequence number\n", filename);
+          free(tag_length);
+          free(tag_ptr);
+          return NULL;
+        }
+
+        tag_length[seq_id-1] = seq_size;
+        tag_ptr[seq_id-1]  = marker->data + icc_offset;
         i++;
       }
       /* copy jpeg comment here to be printed out when EXIF data or comment window is displayed. */
@@ -906,6 +935,13 @@ char *get_icc_profile(char *filename)
       /* No icc markers found */
       return NULL;
     }
+    else if(i != seq_max){
+      /* icc markers are missing */
+      printf("qiv warning: %s has incomplete ICC profile\n", filename);
+      free(tag_length);
+      free(tag_ptr);
+      return NULL;
+    }
     else
     {
       /* Sort the markers and copy them together */
@@ -1103,7 +1139,7 @@ BOOL dpms_enabled = 0;
 void dpms_check()
 {
   int not_needed;
-  Display * disp = GDK_DISPLAY();
+  Display * disp = gdk_x11_display_get_xdisplay(gdk_display_get_default());
   CARD16 power_level;
 
   if(DPMSQueryExtension(disp, &not_needed, &not_needed))
@@ -1119,7 +1155,7 @@ void dpms_enable()
 {
   if(dpms_enabled)
   {
-    DPMSEnable(GDK_DISPLAY());
+    DPMSEnable(gdk_x11_display_get_xdisplay(gdk_display_get_default()));
   }
 }
 
@@ -1127,6 +1163,6 @@ void dpms_disable()
 {
   if(dpms_enabled)
   {
-    DPMSDisable(GDK_DISPLAY());
+    DPMSDisable(gdk_x11_display_get_xdisplay(gdk_display_get_default()));
   }
 }
