@@ -23,11 +23,10 @@ void *xmalloc(size_t size)
 	if (allocated == NULL) {
 		fprintf(stderr, "Error:  Insufficient memory "
 # if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)
-				"(attempt to malloc %zu bytes)\n",
+				"(attempt to malloc %zu bytes)\n", size);
 #else
-				"(attempt to malloc %u bytes)\n",
+				"(attempt to malloc %u bytes)\n", (unsigned int) size);
 #endif
-				 (unsigned int) size);
 		exit(EXIT_FAILURE);
 	}
 
@@ -41,11 +40,10 @@ void *xcalloc(size_t num, size_t size)
 	if (allocated == NULL) {
 		fprintf(stderr, "Error:  Insufficient memory "
 # if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)
-				"(attempt to calloc %zu bytes)\n",
+				"(attempt to calloc %zu bytes)\n", size);
 #else
-				"(attempt to calloc %u bytes)\n",
+				"(attempt to calloc %u bytes)\n", (unsigned int) size);
 #endif
-				 (unsigned int) size);
 		exit(EXIT_FAILURE);
 	}
 
@@ -66,11 +64,10 @@ void *xrealloc(void *ptr, size_t size)
 	if (allocated == NULL) {
 		fprintf(stderr, "Error:  Insufficient memory "
 # if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)
-				"(attempt to realloc %zu bytes)\n",
+				"(attempt to realloc %zu bytes)\n", size);
 #else
-				"(attempt to realloc %u bytes)\n",
+				"(attempt to realloc %u bytes)\n", (unsigned int) size);
 #endif
-				 (unsigned int) size);
 		exit(EXIT_FAILURE);
 	}
 
